//////////////////////////////////////////////////////////////////////
/*****************************************************************/
// This file is a part of the E-XD++ MFC Extension Class.
// 2001-201? ucancode.net Software, All Rights Reserved.
// This is only define for UCanCode Software E-XD++ Library.
//
// UCanCode SOFTWARE GRANTS TO YOU (ONE SOFTWARE PROGRAMMER) THE
// LIMITED RIGHT TO USE THIS SOFTWARE ON A SINGLE COMPUTER.  
//  THESE SOURCE FILE ARE CONSIDERED CONFIDENTIONAL AND ARE 
// THE PROPERTY OF UCanCode SOFTWARE AND ARE NOT TO BE RE-DISTRIBUTED
// BY ANY MEANS WHATSOEVER WITHOUT THE EXPRESSED WRITTEN CONSENT OF 
// UCanCode SOFTWARE.
//
// You can contact us.
// Support@ucancode.net
// http://www.ucancode.net
/********************************************************************/
// FOPGDIPlusProxy.cpp : Defines the class behaviors for the application.
//
  
#include "stdafx.h"
#include "FOPGDIPlusProxy.h"
#include "FOGlobals.h"
#include "FODrawHelper.h"
#include "FOPGDIPlusHelper.h"
#include "FOImagePNG.h"
#include "FOMatrix.h"

#include <math.h>
#include <io.h>
  

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
#define FOP_CONST_ALPHA 255


IMPLEMENT_DYNCREATE(CFOPGDIPlusProxy, CFOPVisualProxy)

inline Gdiplus::PointF X_FPoint(double x, double y, BOOL bPixelOffset = FALSE)
{
	Gdiplus::PointF pointGP((Gdiplus::REAL)x, (Gdiplus::REAL)y);
	
	if (bPixelOffset)
	{
		pointGP.X = (Gdiplus::REAL)((int)pointGP.X + (Gdiplus::REAL)0.5);
		pointGP.Y = (Gdiplus::REAL)((int)pointGP.Y + (Gdiplus::REAL)0.5);
	}

	return pointGP;
}

inline Gdiplus::PointF X_FPoint(const FOPFloat& value, BOOL bPixelOffset = FALSE)
{
	return X_FPoint((Gdiplus::REAL)value.x, (Gdiplus::REAL)value.y, bPixelOffset);
}

inline Gdiplus::SizeF X_FSize(const FOPSize& value)
{
	return Gdiplus::SizeF((Gdiplus::REAL)value.cx, (Gdiplus::REAL)value.cy);
}

inline Gdiplus::RectF X_FRect(const FOPRect& value, BOOL bPixelOffset = FALSE)
{
	FOPRect rectGP(value);
	
	if (bPixelOffset)
	{
		rectGP.left = (int)rectGP.left + .5;
		rectGP.right = (int)rectGP.right + .5;
		rectGP.top = (int)rectGP.top + .5;
		rectGP.bottom = (int)rectGP.bottom + .5;
	}
	
	return Gdiplus::RectF((Gdiplus::REAL)rectGP.left, (Gdiplus::REAL)rectGP.top, (Gdiplus::REAL)rectGP.Width(), (Gdiplus::REAL)rectGP.Height());
}

double xfop_normalize_deg(double value)
{
	if(fabs(value) >= 360.0)
    {
        value -= ((long)(value / 360)) * 360;
    }
	
    if(value < 0.0)
    {
        value += 360.0;
    }
	
    return value;
}

inline double xfop_rad2deg(double value)
{
	return value * 180.0 / F_PI;
}

inline double xfop_angle (double dx, double dy)
{
	if (dy == 0)
	{
		return (dx < 0.0) ? -F_PI : 0.0;
	}
	
	return atan2 (dy, dx);
}

static Gdiplus::GraphicsPath* X_FGenPath(const FOPComplexGeometry& geometry)
{
	Gdiplus::GraphicsPath* pPath = new Gdiplus::GraphicsPath;
	if (pPath == NULL)
	{
		ASSERT(FALSE);
		return NULL;
	}
	if (geometry.IsKindOf(RUNTIME_CLASS(FOPComplexGeometry)))
	{
		FOPComplexGeometry& complexGeometry = (FOPComplexGeometry&)geometry;
		const CObArray& arSegments = complexGeometry.GetSegments();
		int nCount = (int)arSegments.GetSize();

		FOPFloat ptCurr = complexGeometry.GetStartPoint();

		pPath->SetFillMode(Gdiplus::FillModeAlternate); // Gdiplus::FillModeWinding

		pPath->StartFigure();

		for (int i = 0; i < nCount; i++)
		{
			CObject* pSegment = arSegments[i];
			ASSERT_VALID(pSegment);

			if (pSegment->IsKindOf(RUNTIME_CLASS(FOPLineSegment)))
			{
				FOPLineSegment* pLineSegment = (FOPLineSegment*)pSegment;

				pPath->AddLine(X_FPoint(ptCurr), X_FPoint(pLineSegment->m_Point));

				ptCurr = pLineSegment->m_Point;
			}
			else if (pSegment->IsKindOf(RUNTIME_CLASS(FOPBezierSegment)))
			{
				FOPBezierSegment* pBezierSegment = (FOPBezierSegment*)pSegment;

				pPath->AddBezier(X_FPoint(ptCurr), X_FPoint(pBezierSegment->m_Point1),
					X_FPoint(pBezierSegment->m_Point2), X_FPoint(pBezierSegment->m_Point3));

				ptCurr = pBezierSegment->m_Point3;
			}
			else if (pSegment->IsKindOf(RUNTIME_CLASS(FOPArcSegment)))
			{
				//ASSERT(FALSE);
				FOPArcSegment* pArcSegment = (FOPArcSegment*)pSegment;

				Gdiplus::GraphicsPath* pPathArc = NULL;
				if (pArcSegment->m_dblRotationAngle != 0.0)
				{
					pPathArc = new Gdiplus::GraphicsPath;
				}

				BOOL bIsLargeArc = pArcSegment->m_bIsLargeArc;

				double rX;
				double rY;

				FOPFloat ptCenter = pArcSegment->GetArcCenter(ptCurr, bIsLargeArc, rX, rY);
				if (rX == 0.0 || rY == 0.0)
				{
					break;
				}

				double startAngle  = xfop_normalize_deg(xfop_rad2deg(xfop_angle((ptCurr.x - ptCenter.x), (ptCenter.y - ptCurr.y))));
				double finishAngle = xfop_normalize_deg(xfop_rad2deg(xfop_angle((pArcSegment->m_Point.x - ptCenter.x), (ptCenter.y - pArcSegment->m_Point.y))));
				double sweepAngle  = finishAngle - startAngle;

				if (pArcSegment->m_bIsClockwise)
				{
					if (sweepAngle > 0.0)
					{
						sweepAngle -= 360.0;
					}
				}
				else
				{
					if (sweepAngle < 0.0)
					{
						sweepAngle += 360.0;
					}
				}

				if (pPathArc != NULL)
				{
					startAngle += pArcSegment->m_dblRotationAngle;

					pPathArc->AddArc
							(
								Gdiplus::RectF((Gdiplus::REAL)(-rX), (Gdiplus::REAL)(-rY), (Gdiplus::REAL)(rX * 2.0), (Gdiplus::REAL)(rY * 2.0)),
								(Gdiplus::REAL)(360.0 - startAngle),
								-(Gdiplus::REAL)sweepAngle
							);
					Gdiplus::Matrix m;
					m.Rotate((Gdiplus::REAL)pArcSegment->m_dblRotationAngle);
					m.Translate((Gdiplus::REAL)(ptCenter.x), (Gdiplus::REAL)(ptCenter.y), Gdiplus::MatrixOrderAppend);
					pPathArc->Transform(&m);

					pPath->AddPath(pPathArc, TRUE);
				}
				else
				{
					pPath->AddArc
							(
								Gdiplus::RectF((Gdiplus::REAL)(ptCenter.x - rX), (Gdiplus::REAL)(ptCenter.y - rY), (Gdiplus::REAL)(rX * 2.0), (Gdiplus::REAL)(rY * 2.0)),
								(Gdiplus::REAL)(360.0 - startAngle),
								-(Gdiplus::REAL)sweepAngle
							);
				}

				ptCurr = pArcSegment->m_Point;
			}
		}

		if (complexGeometry.IsClosed())
		{
			pPath->CloseFigure();
		}
	}
	
	return pPath;
}

CFOPGDIPlusProxy::CFOPGDIPlusProxy()
{
	m_nTransparent = FOP_CONST_ALPHA;
	m_bWithGDIPlus = FALSE;
}

CFOPGDIPlusProxy::~CFOPGDIPlusProxy()
{

}

/////////////////////////////////////////////////////////////////////////////
// CFOPGDIPlusProxy diagnostics

#ifdef _DEBUG
void CFOPGDIPlusProxy::AssertValid() const
{
	CObject::AssertValid();
}

void CFOPGDIPlusProxy::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
}
#endif //_DEBUG

void CFOPGDIPlusProxy::PrepareTransX(const BYTE &nTrans)
{
	m_nTransparent = (BYTE)nTrans;
}

void CFOPGDIPlusProxy::Create(const BOOL &bWithGDIPlus)
{
	m_bWithGDIPlus = bWithGDIPlus;
}

void CFOPGDIPlusProxy::AttachDC(CDC* pDC)
{
	pDC;
}

void CFOPGDIPlusProxy::ReleaseC()
{
//	m_bWithGDIPlus = FALSE;
}

BOOL CFOPGDIPlusProxy::GetGDIPlus()
{
	return m_bWithGDIPlus;
}

void CFOPGDIPlusProxy::FillShadowCloseBezier(CDC *pDC,LPPOINT points,int nCount,COLORREF crStart, COLORREF crEnd, 
											 UINT nFillType,CPoint ptOffset,CBrush *pBrush)
{
	FOPPoint ptNew;
	int i = 0;
	for (i = 0; i < nCount; i++)
	{
		ptNew = CPoint(points[i]);
		ptNew += ptOffset;
		points[i] = CPoint(ptNew);
	}

	FillCloseBezier(pDC,points, nCount,
		crStart,
		crEnd,
		nFillType,pBrush);
	
	for (i = 0; i < nCount; i++)
	{
		ptNew = CPoint(points[i]);
		ptNew -= ptOffset;
		points[i] = CPoint(ptNew);
	}
}

void CFOPGDIPlusProxy::PrepareGraphics(Graphics *pGraphics)
{
	pGraphics->SetCompositingMode( CompositingModeSourceOver );
//	pGraphics->SetCompositingQuality( CompositingQualityHighQuality );
	pGraphics->SetSmoothingMode( HokGdiPlusG::m_smoothingMode );
	pGraphics->SetPixelOffsetMode(PixelOffsetModeHalf);
	pGraphics->SetPageUnit( UnitPixel );

}

class CFOPA2W
{
public:
	CFOPA2W(LPCSTR lpstr);
	~CFOPA2W();
	operator LPCWSTR() const;
	
protected:
	LPWSTR malpStr;
};

CFOPA2W::CFOPA2W(LPCSTR lptstr)
{
	int nCodePage = GetACP();

	int nDestLen = MultiByteToWideChar(nCodePage, 0, lptstr, -1, NULL, 0);
	
	malpStr = new WCHAR[nDestLen + 1];
	MultiByteToWideChar(nCodePage, 0, lptstr, -1, malpStr, nDestLen);
	malpStr[nDestLen] = 0;
}

CFOPA2W::~CFOPA2W()
{
	delete[] malpStr;
}

CFOPA2W::operator LPCWSTR() const
{
	return malpStr;
}


#ifdef _UNICODE
#define FOP_CT2CW(lp) ((LPCWSTR)(LPCTSTR)(lp))
#else
#define FOP_CT2CW(lp) ((LPCWSTR)CFOPA2W((LPCTSTR)(lp)))
#endif

void CFOPGDIPlusProxy::DoDrawTextOut(CDC *pDC,const CPoint &ptText,const CString &strText,const int &nLengthX)
{
// 	if(gfxData.m_bIsHMIMode)
// 	{
// 		CFOPVisualProxy::DoDrawTextOut(pDC, ptText, strText, nLengthX);
// 			return;
// 	}

	if(strText.IsEmpty())
	{
		return;
	}

	//pBrush;
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		if(gfxData.m_bWithGDIPlusText)
		{
			CFOPVisualProxy::DoDrawTextOut(pDC, ptText, strText, nLengthX);
			return;
		}
		
	}

	int nLength = nLengthX;

	if(nLength <= 0)
	{
		nLength = strText.GetLength();
	}
	
	Gdiplus::Font* pFontG = CFOPHokFontCache::GetInstance()->GetFont(pDC->GetCurrentFont(), pDC);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	CFont *pFont = pDC->GetCurrentFont();
	
	LOGFONT lf;
	ZeroMemory(&lf, sizeof(lf));
	pFont->GetLogFont(&lf);

	if (NULL != pFontG)
	{
		UINT uFormat = DT_LEFT|DT_TOP|DT_SINGLELINE;
		COLORREF crText = pDC->GetTextColor();
		Brush *pBrushG = NULL;
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(1, crText, crText, m_nTransparent, m_nTransparent);
		StringFormat stringFormat(StringFormat::GenericTypographic());
		stringFormat.SetLineAlignment(uFormat & DT_VCENTER ? StringAlignmentCenter: StringAlignmentNear);
		stringFormat.SetAlignment(uFormat & DT_CENTER ? StringAlignmentCenter : uFormat & DT_RIGHT ? StringAlignmentFar : StringAlignmentNear);
		
		
		graphics.TranslateTransform(ptText.x, ptText.y);
		graphics.RotateTransform(-lf.lfOrientation/10);
		graphics.TranslateTransform(-ptText.x, -ptText.y);
		
		PointF ptx((REAL)ptText.x, (REAL)ptText.y);
		FOP_DrawString(&graphics, strText, *pFontG, *pBrushG, ptx, stringFormat);
		_SafeDeletePtr(pBrushG);
	//	_SafeDeletePtr(pFontG);
	}

	//m_nTransparent = 255;
}


void CFOPGDIPlusProxy::FillCloseBezier(CDC *pDC,LPPOINT points,int nCount,COLORREF crStart,COLORREF crEnd,
									  UINT nFillType,CBrush *pBrush)
{
	//pBrush;
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillCloseBezier(pDC, points, nCount, crStart, crEnd, nFillType, pBrush);
		return;
	}

	if(nFillType >= 92)
	{
		if(1)//if(nFillType != FOP_CUSTOM_GRADIENT_TYPE)
			CFOPVisualProxy::FillCloseBezier(pDC, points, nCount, crStart, crEnd,nFillType, pBrush);
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),255);

		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		GraphicsPath localPath;
		localPath.AddBeziers((const Point *)points, nCount);
		localPath.CloseFigure();
		
		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
		{
			CRect rect = CFODrawHelper::GetBoundRect(points, nCount);
			RectF r(rect.left, rect.top, rect.Width(), rect.Height());
			Brush *pBrSolid = NULL;
			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
				pBrSolid = CreatePathBrush(r);
			else
				pBrSolid = CreateRadialBrush(r);
			
			if(pBrSolid != NULL)
				graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		if (NULL != pPenG)
			graphics.DrawPath(pPenG, &localPath);
		
		_SafeDeletePtr(pPenG);
	 
		//m_nTransparent = 255;
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = CFODrawHelper::GetBoundRect(points, nCount);
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	GraphicsPath localPath;
	localPath.AddBeziers((const Point *)points, nCount);
	localPath.CloseFigure();

	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPath(pBrushG, &localPath);
	}
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &localPath);

 
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::FillPoly(CDC *pDC,LPPOINT points,int nCount,COLORREF crStart,COLORREF crEnd,UINT nFillType,
							   CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillPoly(pDC, points, nCount, crStart, crEnd, nFillType, pBrush);
		return;
	}

	if(nFillType >= 92)
	{
		if(1)//if(nFillType != FOP_CUSTOM_GRADIENT_TYPE)
			CFOPVisualProxy::FillPoly(pDC, points, nCount, crStart, crEnd,nFillType, pBrush);
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
	
		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
		{
			CRect rect = CFODrawHelper::GetBoundRect(points, nCount);
			RectF r(rect.left, rect.top, rect.Width(), rect.Height());
			Brush *pBrSolid = NULL;
			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
				pBrSolid = CreatePathBrush(r);
			else
				pBrSolid = CreateRadialBrush(r);
			
			if(pBrSolid != NULL)
				graphics.FillPolygon(pBrSolid, (const Point *)points, nCount);
			_SafeDeletePtr(pBrSolid);
		}

		if (NULL != pPenG)
			graphics.DrawPolygon(pPenG, (const Point *)points, nCount);
		//m_nTransparent = 255;
	 
		_SafeDeletePtr(pPenG);
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = CFODrawHelper::GetBoundRect(points, nCount);
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPolygon(pBrSolid, (const Point *)points, nCount);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPolygon(pBrSolid, (const Point *)points, nCount);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPolygon(pBrushG, (const Point *)points, nCount);
	}

	if (NULL != pPenG)
		graphics.DrawPolygon(pPenG, (const Point *)points, nCount);

	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

BOOL CFOPGDIPlusProxy::FillPolyPolygon(CDC *pDC,LPPOINT points,LPINT lpPolyCounts,int nCount,const CRect &rcPos,
									  COLORREF crStart,COLORREF crEnd,UINT nFillType,CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillPolyPolygon(pDC, points, lpPolyCounts, nCount, rcPos, crStart, crEnd, nFillType, pBrush);
		return TRUE;
	}

	if(nFillType >= 92)
	{
		if(1)//if(nFillType != FOP_CUSTOM_GRADIENT_TYPE)
			CFOPVisualProxy::FillPolyPolygon(pDC, points,lpPolyCounts, nCount, rcPos, crStart, crEnd, nFillType, pBrush);
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		GraphicsPath localPath;
		
		int i = 0;
		LPPOINT localPt = points;
		for (; i<nCount; i++)
		{
			localPath.AddPolygon((const Point *)localPt, lpPolyCounts[i]);
			localPath.CloseFigure();
			localPt += lpPolyCounts[i];
		}
		
		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
		{
			CRect rect = CFODrawHelper::GetBoundRect(points, nCount);
			RectF r(rect.left, rect.top, rect.Width(), rect.Height());
			Brush *pBrSolid = NULL;
			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
				pBrSolid = CreatePathBrush(r);
			else
				pBrSolid = CreateRadialBrush(r);
			
			if(pBrSolid != NULL)
				graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		
		if (NULL != pPenG)
			graphics.DrawPath(pPenG, &localPath);

		_SafeDeletePtr(pPenG);
		return TRUE;
	}


	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = CFODrawHelper::GetBoundRect(points, nCount);
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	GraphicsPath localPath;

	int i = 0;
	LPPOINT localPt = points;
	for (; i<nCount; i++)
	{
		localPath.AddPolygon((const Point *)localPt, lpPolyCounts[i]);
		localPath.CloseFigure();
		localPt += lpPolyCounts[i];
	}

	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPath(pBrushG, &localPath);
	}

	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &localPath);

	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;

	return TRUE;
}

void CFOPGDIPlusProxy::FillBezier(CDC *pDC,LPPOINT points,int nCount,COLORREF crStart,COLORREF crEnd,
								 UINT nFillType,CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillBezier(pDC, points, nCount, crStart, crEnd, nFillType, pBrush);
		return;
	}

	if(nFillType >= 92)
	{
		if(1)//if(nFillType != FOP_CUSTOM_GRADIENT_TYPE)
			CFOPVisualProxy::FillBezier(pDC, points,nCount, crStart, crEnd,nFillType, pBrush);
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		GraphicsPath localPath;
		localPath.AddBeziers((const Point *)points, nCount);
		localPath.CloseFigure();
		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
		{
			CRect rect = CFODrawHelper::GetBoundRect(points, nCount);
			RectF r(rect.left, rect.top, rect.Width(), rect.Height());
			Brush *pBrSolid = NULL;
			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
				pBrSolid = CreatePathBrush(r);
			else
				pBrSolid = CreateRadialBrush(r);
			
			if(pBrSolid != NULL)
				graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		if (NULL != pPenG)
			graphics.DrawPath(pPenG, &localPath);
 
		_SafeDeletePtr(pPenG);
		//m_nTransparent = 255;
		
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = CFODrawHelper::GetBoundRect(points, nCount);
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	GraphicsPath localPath;
	localPath.AddBeziers((const Point *)points, nCount);
	localPath.CloseFigure();

	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart,RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd,RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPath(pBrushG, &localPath);
	}

	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &localPath);


	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);

	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::FillCanShape(CDC *pDC,LPPOINT pointsOld,int nCount,COLORREF crStart,COLORREF crEnd,
								   UINT nFillType,CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillCanShape(pDC, pointsOld, nCount, crStart, crEnd, nFillType, pBrush);
		return;
	}

	if(nFillType >= 92)
	{
		CFOPVisualProxy::FillCanShape(pDC, pointsOld,nCount, crStart, crEnd,nFillType, pBrush);
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		GraphicsPath localPath;
		
		Point ptLine[5] = {Point(pointsOld[19].x, pointsOld[19].y),
			Point(pointsOld[12].x, pointsOld[12].y), Point(pointsOld[6].x, pointsOld[6].y),
			Point(pointsOld[13].x, pointsOld[13].y), Point(pointsOld[19].x, pointsOld[19].y)};
		localPath.AddLines(ptLine, 5);
		localPath.CloseFigure();
		
		Point ptBezier[7] = {Point(pointsOld[13].x, pointsOld[13].y),
			Point(pointsOld[14].x, pointsOld[14].y), Point(pointsOld[15].x, pointsOld[15].y),
			Point(pointsOld[16].x, pointsOld[16].y), Point(pointsOld[17].x, pointsOld[17].y),
			Point(pointsOld[18].x, pointsOld[18].y), Point(pointsOld[19].x, pointsOld[19].y)};
		localPath.AddBeziers(ptBezier, 7);
		localPath.CloseFigure();

		if (NULL != pPenG)
			graphics.DrawPath(pPenG, &localPath);
		
	 
		_SafeDeletePtr(pPenG);
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = CFODrawHelper::GetBoundRect(pointsOld, nCount);
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	GraphicsPath localPath;

	Point ptLine[5] = {Point(pointsOld[19].x, pointsOld[19].y),
		Point(pointsOld[12].x, pointsOld[12].y), Point(pointsOld[6].x, pointsOld[6].y),
		Point(pointsOld[13].x, pointsOld[13].y), Point(pointsOld[19].x, pointsOld[19].y)};
	localPath.AddLines(ptLine, 5);
	localPath.CloseFigure();

	Point ptBezier[7] = {Point(pointsOld[13].x, pointsOld[13].y),
		Point(pointsOld[14].x, pointsOld[14].y), Point(pointsOld[15].x, pointsOld[15].y),
		Point(pointsOld[16].x, pointsOld[16].y), Point(pointsOld[17].x, pointsOld[17].y),
		Point(pointsOld[18].x, pointsOld[18].y), Point(pointsOld[19].x, pointsOld[19].y)};
	localPath.AddBeziers(ptBezier, 7);
	localPath.CloseFigure();

	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart,RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd,RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &localPath);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPath(pBrushG, &localPath);
	}
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &localPath);


	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::DoDrawTabOrder(CDC *pDC,const CRect &rcPos,const COLORREF &crColor,CString strText)
{
	CRect rcText = rcPos;
	CFODrawHelper::FillRectFast(pDC, rcText,crColor);	
	XDDraw()->GDIText(pDC, strText,&rcText,DT_CENTER|DT_VCENTER|DT_SINGLELINE);
}

#define Hok_PI 3.141592653586f
void CFOPGDIPlusProxy::DoDrawArcLine(CDC *pDC,const CRect &rect,const CPoint &ptStart,const CPoint &ptEnd)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DoDrawArcLine(pDC, rect, ptStart, ptEnd);
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	RectF theRect(REAL(rect.left), REAL(rect.top), REAL(rect.Width()), REAL(rect.Height()));
	PointF center(theRect.X + theRect.Width/2, theRect.Y + theRect.Height/2);

	double startAngle = 180 * atan2((double)(ptStart.y - center.Y), (double)(ptStart.x - center.X))/Hok_PI;
	double endAngle = 180 * atan2((double)(ptEnd.y - center.Y), (double)(ptEnd.x - center.X))/Hok_PI;
	startAngle = (startAngle>0)? startAngle : startAngle+360;
	endAngle = (endAngle>0)? endAngle : endAngle+360;

	double dAngle = endAngle - startAngle;

	if (NULL != pPenG)
	{
		if (dAngle > 0) 
			graphics.DrawArc(pPenG, theRect, REAL(startAngle), REAL(dAngle)-360);
		else if(dAngle == 0.0)
			graphics.DrawArc(pPenG, theRect, REAL(startAngle), REAL(360));
		else
			graphics.DrawArc(pPenG, theRect, REAL(startAngle), REAL(dAngle));
	}
 
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}
 

void CFOPGDIPlusProxy::DoDrawLine(CDC *pDC,const CPoint &ptStart,const CPoint &ptEnd)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DoDrawLine(pDC, ptStart, ptEnd);
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	if (NULL != pPenG)
		graphics.DrawLine(pPenG, Point(ptStart.x, ptStart.y), Point(ptEnd.x, ptEnd.y));
 
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::DoDrawBezierLine(CDC *pDC,LPPOINT points,int nCount)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DoDrawBezierLine(pDC, points, nCount);
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	if (NULL != pPenG)
		graphics.DrawBeziers(pPenG, (const Point*)points, nCount);
 
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::DoDrawCustomTrackPort(CDC *pDC,const CPoint &ptPort,const int &nPortWidth,const int &nPortHeight)
{
	pDC;
	ptPort;
	nPortWidth;
	nPortHeight;
}

void CFOPGDIPlusProxy::DoDrawCustomPort(CDC *pDC,const CPoint &ptPort,const int &nPortWidth,const int &nPortHeight,
	COLORREF crStart, COLORREF crEnd, int nBrushType,int nPortType)
{
	pDC;
	ptPort;
	nPortWidth;
	nPortHeight;
	crStart;
	crEnd;
	nBrushType;
	nPortType;
}

void CFOPGDIPlusProxy::DoDrawPolyLine(CDC *pDC,LPPOINT points,int nCount)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DoDrawPolyLine(pDC, points, nCount);
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	if (NULL != pPenG)
		graphics.DrawLines(pPenG, (const Point *)points, nCount);
 
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

// Draw polygon
void CFOPGDIPlusProxy::DoDrawPolygon(CDC *pDC,LPPOINT points,int nCount)
{

	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		pDC->Polygon(points, nCount);
		return;
	}
	
	if(m_nTransparent == 255)
	{
		pDC->Polygon(points, nCount);

		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		if (NULL != pPenG)
			graphics.DrawPolygon(pPenG, (const Point *)points, nCount);
		
		//m_nTransparent = 255;
		 
		_SafeDeletePtr(pPenG);
		return;
	}

	
}

// Drawing tracking line.
void CFOPGDIPlusProxy::DoDrawPolyPolygon(CDC *pDC,LPPOINT points,LPINT lpPoly,int nCount)
{
	pDC->PolyPolygon(points,lpPoly, nCount);
}

//////////////////////////////////////////////////////////////////////////

void FOPPreparePolyDrawG(  BOOL						bCloseFigures,
                          ULONG 					nPoly,
                          const ULONG* 				pPoints,
                          const fopPointExt* const*	pPtAry,
                          const BYTE* const* 		pFlgAry,
                          POINT* 					pWinPointAry,
                          BYTE* 					pWinFlagAry		)
{
    ULONG nCurrPoly;
    for( nCurrPoly=0; nCurrPoly<nPoly; ++nCurrPoly )
    {
        const POINT* pCurrPoint = reinterpret_cast<const POINT*>( *pPtAry++ );
        const BYTE* pCurrFlag = *pFlgAry++;
        const ULONG nCurrPoints = *pPoints++;
        const BOOL bHaveFlagArray( (BOOL)pCurrFlag );
        ULONG nCurrPoint;

        if( nCurrPoints )
        {
            *pWinPointAry++ = *pCurrPoint++;
            *pWinFlagAry++  = PT_MOVETO;
            ++pCurrFlag;

            for( nCurrPoint=1; nCurrPoint<nCurrPoints; )
            {
                if( bHaveFlagArray &&
                    ( nCurrPoint + 2 ) < nCurrPoints )
                {
                    BYTE P4( pCurrFlag[ 2 ] );

                    if( ( FOP_SIM_POLY_CONTROL == pCurrFlag[ 0 ] ) &&
                        ( FOP_SIM_POLY_CONTROL == pCurrFlag[ 1 ] ) &&
                        ( FOP_SIM_POLY_NORMAL == P4 || FOP_SIM_POLY_SMOOTH == P4 || FOP_SIM_POLY_SYMMTR == P4 ) )
                    {
                        *pWinPointAry++ = *pCurrPoint++;
                        *pWinFlagAry++  = PT_BEZIERTO;

                        *pWinPointAry++ = *pCurrPoint++;
                        *pWinFlagAry++  = PT_BEZIERTO;

                        *pWinPointAry++ = *pCurrPoint++;
                        *pWinFlagAry++  = PT_BEZIERTO;

                        nCurrPoint += 3;
                        pCurrFlag += 3;
                        continue;
                    }
                }

                *pWinPointAry++ = *pCurrPoint++;
                *pWinFlagAry++  = PT_LINETO;
                ++pCurrFlag;
                ++nCurrPoint;
            }

            if( bCloseFigures )
			{
                pWinFlagAry[-1] |= PT_CLOSEFIGURE;
			}

        }
    }
}


void CFOPGDIPlusProxy::FOPDoRenderPathG(CDC *pDC, ULONG nPoints, const BYTE &nTrans, 
										const fopPointExt* pPtAry, const BYTE* pFlgAry )
{
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),nTrans);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	fopPointExt pPtMove;

    if( nPoints )
    {
		pPtMove.nLX = pPtAry[0].nLX;
		pPtMove.nLY = pPtAry[0].nLY;
        ++pFlgAry;

        for(USHORT i=1; i<nPoints; ++i,++pFlgAry )
        {
            if( *pFlgAry != FOP_SIM_POLY_CONTROL )
            {
				if (NULL != pPenG)
					graphics.DrawLine(pPenG, Point(pPtMove.nLX, pPtMove.nLY), Point(pPtAry[i].nLX, pPtAry[i].nLY));

				pPtMove.nLX = pPtAry[i].nLX;
				pPtMove.nLY = pPtAry[i].nLY;		
            }
            else if( nPoints - i > 2 )
            {
				Point points[4];
				points[0] = Point(pPtMove.nLX,pPtMove.nLY);
				points[1] = Point(pPtAry[i].nLX,pPtAry[i].nLY);
				points[2] = Point(pPtAry[i+1].nLX,pPtAry[i+1].nLY);
				points[3] = Point(pPtAry[i+2].nLX,pPtAry[i+2].nLY);

				if (NULL != pPenG)
					graphics.DrawBezier(pPenG, 
					points[0],
					points[1],
					points[2],
					points[3]);

                i += 2; pFlgAry += 2;
				pPtMove.nLX = points[3].X;
				pPtMove.nLY = points[3].Y;		
			}
        }
	}

}

/*

void CFOPGDIPlusProxy::FOPPolyDrawG(CDC *pDC, const BYTE &nTrans, CONST LPPOINT lppt, CONST LPBYTE lpbTypes, int cCount,
			   BOOL bFill, COLORREF crStart, COLORREF crEnd, UINT nFillType)
{
//	HDC dc = pDC->m_hDC;
	int nIndex;
	LPPOINT pptLastMoveTo = NULL;
	POINT ptStart;
	ptStart.x = 0;
	ptStart.y = 0;
	GraphicsPath localPath;
	GraphicsPath AllPath;

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),nTrans);

	// for each of the points we have...
	for (nIndex = 0; nIndex < cCount; nIndex++)
	{
		switch(lpbTypes[nIndex])
		{
		case PT_MOVETO:
			{
				localPath.StartFigure();
				if (pptLastMoveTo != NULL && nIndex > 0)
				{
//					localPath.AddLine(Point(ptStart.x,ptStart.y),Point(pptLastMoveTo->x, pptLastMoveTo->y));
					//				LineTo(dc, );
					ptStart.x = pptLastMoveTo->x;
					ptStart.y = pptLastMoveTo->y;
				}
				
				ptStart = lppt[nIndex];
				//			MoveToEx(dc, lppt[nIndex].x, lppt[nIndex].y, NULL);
				pptLastMoveTo = &lppt[nIndex];
			}
			break;
			
		case PT_LINETO | PT_CLOSEFIGURE:
			{
				//			LineTo(dc, lppt[nIndex].x, lppt[nIndex].y);
				localPath.AddLine(Point(ptStart.x,ptStart.y),Point(lppt[nIndex].x, lppt[nIndex].y));
				ptStart = lppt[nIndex];
				if (pptLastMoveTo != NULL)
				{
					//				LineTo(dc, pptLastMoveTo->x, pptLastMoveTo->y);
					localPath.AddLine(Point(ptStart.x,ptStart.y),Point(pptLastMoveTo->x, pptLastMoveTo->y));
					ptStart.x = pptLastMoveTo->x;
					ptStart.y = pptLastMoveTo->y;
				}
				localPath.CloseFigure();
				
				AllPath.AddPath(&localPath,false);

				localPath.Reverse();
				localPath.Reset();
				
				pptLastMoveTo = NULL;

			}
			break;
			
		case PT_LINETO:
			{
				//			LineTo(dc, lppt[nIndex].x, lppt[nIndex].y);
				localPath.AddLine(Point(ptStart.x,ptStart.y),Point(lppt[nIndex].x, lppt[nIndex].y));
				ptStart = lppt[nIndex];
			}
			break;
			
		case PT_BEZIERTO | PT_CLOSEFIGURE:
			{
				Point points[4];
				points[0] = Point(ptStart.x,ptStart.y);
				points[1] = Point(lppt[nIndex].x,lppt[nIndex].y);
				points[2] = Point(lppt[nIndex + 1].x,lppt[nIndex + 1].y);
				points[3] = Point(lppt[nIndex + 2].x,lppt[nIndex + 2].y);
				localPath.AddBezier(points[0],points[1],points[2],points[3]);
				//			PolyBezierTo(dc, &lppt[nIndex], 3);
				nIndex += 2;
				ptStart = lppt[nIndex];
				if (pptLastMoveTo != NULL)
				{
					//				LineTo(dc, pptLastMoveTo->x, pptLastMoveTo->y);
					localPath.AddLine(Point(ptStart.x,ptStart.y),Point(pptLastMoveTo->x, pptLastMoveTo->y));
					ptStart.x = pptLastMoveTo->x;
					ptStart.y = pptLastMoveTo->y;
				}
				
				localPath.CloseFigure();

				AllPath.AddPath(&localPath,false);

				localPath.Reverse();
				localPath.Reset();

				pptLastMoveTo = NULL;
			}
			break;
			
		case PT_BEZIERTO:
			{
				Point points[4];
				points[0] = Point(ptStart.x,ptStart.y);
				points[1] = Point(lppt[nIndex].x,lppt[nIndex].y);
				points[2] = Point(lppt[nIndex + 1].x,lppt[nIndex + 1].y);
				points[3] = Point(lppt[nIndex + 2].x,lppt[nIndex + 2].y);
				localPath.AddBezier(points[0],points[1],points[2],points[3]);
				//			PolyBezierTo(dc, &lppt[nIndex], 3);
				nIndex += 2;
				ptStart = lppt[nIndex];
			}
			break;
		}
	}

	if (pptLastMoveTo != NULL && nIndex > 1)
	{
//		LineTo(dc, pptLastMoveTo->x, pptLastMoveTo->y);
		localPath.AddLine(Point(ptStart.x,ptStart.y),Point(pptLastMoveTo->x, pptLastMoveTo->y));
		localPath.CloseFigure();
		AllPath.AddPath(&localPath,false);
	}
	
	Graphics graphics(pDC->m_hDC);
	graphics.SetSmoothingMode(HokGdiPlusG::m_smoothingMode);
	graphics.SetPageUnit(UnitPixel);

	if(bFill == TRUE)
	{
		Brush *pBrushG = NULL;
		if ( 41 < nFillType && nFillType <62  )
		{
			CRect rc = CFODrawHelper::GetBoundRect(lppt, cCount);
			pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, nTrans, nTrans);
		}
		else
		{
			pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, nTrans, nTrans);
		}

		if (NULL != pBrushG)
		{
			if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
			{
				Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart,RGB(0,0,0), nTrans, nTrans);
				graphics.FillPath(pBrSolid, &AllPath);
			}
			if (nFillType-42 == GRADIENT_ELLIPSE_IN)
			{
				Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd,RGB(0,0,0), nTrans, nTrans);
				graphics.FillPath(pBrSolid, &AllPath);
			}
			graphics.FillPath(pBrushG, &AllPath);
		}

		if ( 41 < nFillType && nFillType <62  )
			_SafeDeletePtr(pBrushG);
	}

	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &AllPath);

	graphics.Flush();

}

*/

void CFOPGDIPlusProxy::FOPPolyDrawG(CDC *pDC, const BYTE &nTrans, CONST LPPOINT lppt, CONST LPBYTE lpbTypes, int cCount,
			   BOOL bFill, COLORREF crStart, COLORREF crEnd, UINT nFillType)
{
//	HDC dc = pDC->m_hDC;
//	return;
	int nIndex;
	LPPOINT pptLastMoveTo = NULL;
	POINT ptStart;
	ptStart.x = 0;
	ptStart.y = 0;
	GraphicsPath localPath;
	GraphicsPath AllPath;

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),nTrans);

	// for each of the points we have...
	for (nIndex = 0; nIndex < cCount; nIndex++)
	{
		switch(lpbTypes[nIndex])
		{
		case PT_MOVETO:
			{
				if (pptLastMoveTo != NULL && nIndex > 0)
				{
					localPath.CloseFigure();
				}
				
				pptLastMoveTo = &lppt[nIndex];
				ptStart = lppt[nIndex];
				
			}
			break;
			
		case PT_LINETO | PT_CLOSEFIGURE:
			{
				localPath.AddLine(Point(ptStart.x,ptStart.y),Point(lppt[nIndex].x, lppt[nIndex].y));
				ptStart = lppt[nIndex];
				if (pptLastMoveTo != NULL)
				{
					localPath.CloseFigure();
					AllPath.AddPath(&localPath,false);
					localPath.Reverse();
					localPath.Reset();
					ptStart.x = pptLastMoveTo->x;
					ptStart.y = pptLastMoveTo->y;
				}
				pptLastMoveTo = NULL;

			}
			break;
			
		case PT_LINETO:
			{
				localPath.AddLine(Point(ptStart.x,ptStart.y),Point(lppt[nIndex].x, lppt[nIndex].y));
				ptStart = lppt[nIndex];
			}
			break;
			
		case PT_BEZIERTO | PT_CLOSEFIGURE:
			{
				Point points[4];
				points[0] = Point(ptStart.x,ptStart.y);
				points[1] = Point(lppt[nIndex].x,lppt[nIndex].y);
				points[2] = Point(lppt[nIndex + 1].x,lppt[nIndex + 1].y);
				points[3] = Point(lppt[nIndex + 2].x,lppt[nIndex + 2].y);
				localPath.AddBezier(points[0],points[1],points[2],points[3]);
				nIndex += 2;
				ptStart = lppt[nIndex];
				if (pptLastMoveTo != NULL)
				{
					localPath.CloseFigure();
					AllPath.AddPath(&localPath,false);
					localPath.Reverse();
					localPath.Reset();
					ptStart.x = pptLastMoveTo->x;
					ptStart.y = pptLastMoveTo->y;
				}

				pptLastMoveTo = NULL;
			}
			break;
			
		case PT_BEZIERTO:
			{
				Point points[4];
				points[0] = Point(ptStart.x,ptStart.y);
				points[1] = Point(lppt[nIndex].x,lppt[nIndex].y);
				points[2] = Point(lppt[nIndex + 1].x,lppt[nIndex + 1].y);
				points[3] = Point(lppt[nIndex + 2].x,lppt[nIndex + 2].y);
				localPath.AddBezier(points[0],points[1],points[2],points[3]);
				nIndex += 2;
				ptStart = lppt[nIndex];
			}
			break;
		}
	}

	if (pptLastMoveTo != NULL && nIndex > 1)
	{
		localPath.AddLine(Point(ptStart.x,ptStart.y),Point(pptLastMoveTo->x, pptLastMoveTo->y));
		localPath.CloseFigure();
		AllPath.AddPath(&localPath,false);
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	if(bFill == TRUE )
	{
// 		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
// 		{
// 			CRect rect = CFODrawHelper::GetBoundRect(lppt, cCount);
// 			RectF r(rect.left, rect.top, rect.Width(), rect.Height());
// 			Brush *pBrSolid = NULL;
// 			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
// 				pBrSolid = CreatePathBrush(r);
// 			else
// 				pBrSolid = CreateRadialBrush(r);
// 			
// 			if(pBrSolid != NULL)
// 				graphics.FillPath(pBrSolid, &AllPath);
// 			_SafeDeletePtr(pBrSolid);
// 		}
// 		else
		if(1)
		{
			Brush *pBrushG = NULL;
			if ( 41 < nFillType && nFillType <62  )
			{
				CRect rc = CFODrawHelper::GetBoundRect(lppt, cCount);
				pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, nTrans, nTrans);
			}
			else
			{
				pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, nTrans, nTrans);
			}
			
			if (NULL != pBrushG)
			{
				if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
				{
					Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart,RGB(0,0,0), nTrans, nTrans);
					graphics.FillPath(pBrSolid, &AllPath);
					_SafeDeletePtr(pBrSolid);
				}
				if (nFillType-42 == GRADIENT_ELLIPSE_IN)
				{
					Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd,RGB(0,0,0), nTrans, nTrans);
					graphics.FillPath(pBrSolid, &AllPath);
					_SafeDeletePtr(pBrSolid);
				}
				graphics.FillPath(pBrushG, &AllPath);
			}
			
			_SafeDeletePtr(pBrushG);
 
		}
		
	}
// 	else if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
// 	{
// 		CRect rect = CFODrawHelper::GetBoundRect(lppt, cCount);
// 		RectF r(rect.left, rect.top, rect.Width(), rect.Height());
// 		Brush *pBrSolid = NULL;
// 		if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
// 			pBrSolid = CreatePathBrush(r);
// 		else
// 			pBrSolid = CreateRadialBrush(r);
// 		
// 		if(pBrSolid != NULL)
// 			graphics.FillPath(pBrSolid, &AllPath);
// 		_SafeDeletePtr(pBrSolid);
// 	}

	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &AllPath);

	graphics.Flush();
 
	_SafeDeletePtr(pPenG);
}

BOOL CFOPGDIPlusProxy::ExtPolyDrawLine(CDC *pDC, CONST LPPOINT lppt, CONST LPBYTE lpbTypes, int cCount, const BOOL &bClose)
{

	int nIndex;
	LPPOINT pptLastMoveTo = NULL;
	POINT ptStart;
	ptStart.x = 0;
	ptStart.y = 0;
	GraphicsPath localPath;
	GraphicsPath AllPath;

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	// for each of the points we have...
	for (nIndex = 0; nIndex < cCount; nIndex++)
	{
		switch(lpbTypes[nIndex])
		{
		case PT_MOVETO:
			{
				if (pptLastMoveTo != NULL && nIndex > 0)
				{
					localPath.CloseFigure();
				}
				
				pptLastMoveTo = &lppt[nIndex];
				ptStart = lppt[nIndex];
				
			}
			break;
			
		case PT_LINETO | PT_CLOSEFIGURE:
			{
				localPath.AddLine(Point(ptStart.x,ptStart.y),Point(lppt[nIndex].x, lppt[nIndex].y));
				ptStart = lppt[nIndex];
				if (pptLastMoveTo != NULL)
				{
					localPath.CloseFigure();
					AllPath.AddPath(&localPath,false);
					localPath.Reverse();
					localPath.Reset();
					ptStart.x = pptLastMoveTo->x;
					ptStart.y = pptLastMoveTo->y;
				}
				pptLastMoveTo = NULL;

			}
			break;
			
		case PT_LINETO:
			{
				localPath.AddLine(Point(ptStart.x,ptStart.y),Point(lppt[nIndex].x, lppt[nIndex].y));
				ptStart = lppt[nIndex];
			}
			break;
			
		case PT_BEZIERTO | PT_CLOSEFIGURE:
			{
				Point points[4];
				points[0] = Point(ptStart.x,ptStart.y);
				points[1] = Point(lppt[nIndex].x,lppt[nIndex].y);
				points[2] = Point(lppt[nIndex + 1].x,lppt[nIndex + 1].y);
				points[3] = Point(lppt[nIndex + 2].x,lppt[nIndex + 2].y);
				localPath.AddBezier(points[0],points[1],points[2],points[3]);
				nIndex += 2;
				ptStart = lppt[nIndex];
				if (pptLastMoveTo != NULL)
				{
					localPath.CloseFigure();
					AllPath.AddPath(&localPath,false);
					localPath.Reverse();
					localPath.Reset();
					ptStart.x = pptLastMoveTo->x;
					ptStart.y = pptLastMoveTo->y;
				}

				pptLastMoveTo = NULL;
			}
			break;
			
		case PT_BEZIERTO:
			{
				Point points[4];
				points[0] = Point(ptStart.x,ptStart.y);
				points[1] = Point(lppt[nIndex].x,lppt[nIndex].y);
				points[2] = Point(lppt[nIndex + 1].x,lppt[nIndex + 1].y);
				points[3] = Point(lppt[nIndex + 2].x,lppt[nIndex + 2].y);
				localPath.AddBezier(points[0],points[1],points[2],points[3]);
				nIndex += 2;
				ptStart = lppt[nIndex];
			}
			break;
		}
	}

	if (pptLastMoveTo != NULL && nIndex > 1 && bClose)
	{
		localPath.AddLine(Point(ptStart.x,ptStart.y),Point(pptLastMoveTo->x, pptLastMoveTo->y));
		localPath.CloseFigure();
		AllPath.AddPath(&localPath,false);
	}
	else
	{
		AllPath.AddPath(&localPath,false);
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &AllPath);

	graphics.Flush();
 
	_SafeDeletePtr(pPenG);
	return TRUE;

}

#define FOP_PLOY_STACKBUF		32

BOOL CFOPGDIPlusProxy::DrawNewPolyLineBezier(CDC *pDC, ULONG nPoints, const fopPointExt* pPtAry, const BYTE* pFlgAry )
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DrawNewPolyLineBezier(pDC, nPoints, pPtAry, pFlgAry);
		return TRUE;
	}

// 	if(m_nTransparent == 255)
// 	{
// 		CFOPVisualProxy::DrawNewPolyLineBezier(pDC, nPoints, pPtAry, pFlgAry);
// 	}

    FOPDoRenderPathG(pDC, nPoints,m_nTransparent, pPtAry, pFlgAry );
    return TRUE;
}

BOOL CFOPGDIPlusProxy::ExtPolyDraw(CDC *pDC,const CRect &rcPos, 
										   ULONG nPoints, 
										   POINT* pWinPointAry,
										   BYTE* pWinFlagAry,
										COLORREF crStart,
										COLORREF crEnd,UINT nType,
										CBrush *pBrush )
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::ExtPolyDraw(pDC, rcPos, nPoints,pWinPointAry, pWinFlagAry, 
			crStart, crEnd, nType, pBrush);
		return TRUE;
	}

	if(nType >= 92)
	{
	//	if(nType != FOP_CUSTOM_GRADIENT_TYPE)
		CFOPVisualProxy::ExtPolyDraw(pDC, rcPos, nPoints,pWinPointAry, pWinFlagAry, 
			crStart, crEnd, nType, pBrush);
		FOPPolyDrawG(pDC,m_nTransparent, pWinPointAry, pWinFlagAry, nPoints, FALSE, crStart, crEnd, nType);
		return TRUE;
	}


	FOPPolyDrawG(pDC,m_nTransparent, pWinPointAry, pWinFlagAry, nPoints, TRUE, crStart, crEnd, nType);

	//m_nTransparent = 255;
	return TRUE;
}


void CFOPGDIPlusProxy::S_DrawCloseCurve(CDC *pDC, POINT* pWinPointAry, BYTE* pWinFlagAry, int nPoints,  const CRect &rcPos, 
									   COLORREF crStart,COLORREF crEnd,UINT nFillType,CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::S_DrawCloseCurve(pDC, pWinPointAry, pWinFlagAry, nPoints, rcPos, crStart, crEnd, nFillType, pBrush);
		return;
	}

	if(nFillType >= 92)
	{
		if(1)//if(nFillType != FOP_CUSTOM_GRADIENT_TYPE)
			CFOPVisualProxy::S_DrawCloseCurve(pDC, pWinPointAry, pWinFlagAry, nPoints, rcPos, crStart, crEnd, nFillType, pBrush);

		GraphicsPath path((Point* )pWinPointAry, pWinFlagAry, nPoints);
		
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
		{
			CRect rect = CFODrawHelper::GetBoundRect(pWinPointAry, nPoints);
			RectF r(rect.left, rect.top, rect.Width(), rect.Height());
			Brush *pBrSolid = NULL;
			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
				pBrSolid = CreatePathBrush(r);
			else
				pBrSolid = CreateRadialBrush(r);
			
			if(pBrSolid != NULL)
				graphics.FillPath(pBrSolid, &path);
			_SafeDeletePtr(pBrSolid);
		}
		if (NULL != pPenG)
			graphics.DrawPath(pPenG, &path);

		//m_nTransparent = 255;
	 
		_SafeDeletePtr(pPenG);
		return;
	}
	
	GraphicsPath path((Point* )pWinPointAry, pWinFlagAry, nPoints);

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	
	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = CFODrawHelper::GetBoundRect(pWinPointAry, nPoints);
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &path);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &path);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPath(pBrushG, &path);
	}
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &path);
	
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;

}

BOOL CFOPGDIPlusProxy::FillNewPolygonBezier(CDC *pDC,const CRect &rcPos, ULONG nPoints,
		const fopPointExt* pPtAry, const BYTE* pFlgAry, COLORREF crStart,COLORREF crEnd,UINT nType, CBrush *pBrush )
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillNewPolygonBezier(pDC, rcPos, nPoints,pPtAry, pFlgAry, crStart, crEnd, nType, pBrush);
		return TRUE;
	}

	if(nType >= 92)
	{
	//	if(nType != FOP_CUSTOM_GRADIENT_TYPE)
		CFOPVisualProxy::FillNewPolygonBezier(pDC, rcPos, nPoints,pPtAry, pFlgAry, crStart, crEnd, nType, pBrush);
		POINT	aStackAry1[FOP_PLOY_STACKBUF];
		BYTE	aStackAry2[FOP_PLOY_STACKBUF];
		POINT* 	pWinPointAry;
		BYTE* 	pWinFlagAry;
		if( nPoints > FOP_PLOY_STACKBUF )
		{
			pWinPointAry = new POINT[nPoints];
			pWinFlagAry = new BYTE[nPoints];
		}
		else
		{
			pWinPointAry = aStackAry1;
			pWinFlagAry = aStackAry2;
		}
		
		FOPPreparePolyDrawG(true, 1, &nPoints, &pPtAry, &pFlgAry, pWinPointAry, pWinFlagAry);
		
		FOPPolyDrawG(pDC,m_nTransparent, pWinPointAry, pWinFlagAry, nPoints, FALSE, crStart, crEnd, nType);
		
		if( pWinPointAry != aStackAry1 )
		{
			_SafeDeletePtrArr(pWinPointAry);
			_SafeDeletePtrArr(pWinFlagAry);
		}
		
		return TRUE;

	}

    POINT	aStackAry1[FOP_PLOY_STACKBUF];
    BYTE	aStackAry2[FOP_PLOY_STACKBUF];
    POINT* 	pWinPointAry;
    BYTE* 	pWinFlagAry;
    if( nPoints > FOP_PLOY_STACKBUF )
    {
        pWinPointAry = new POINT[nPoints];
        pWinFlagAry = new BYTE[nPoints];
    }
    else
    {
        pWinPointAry = aStackAry1;
        pWinFlagAry = aStackAry2;
    }

	FOPPreparePolyDrawG(true, 1, &nPoints, &pPtAry, &pFlgAry, pWinPointAry, pWinFlagAry);

	FOPPolyDrawG(pDC,m_nTransparent, pWinPointAry, pWinFlagAry, nPoints, TRUE, crStart, crEnd, nType);

    if( pWinPointAry != aStackAry1 )
    {
		_SafeDeletePtrArr(pWinPointAry);
		_SafeDeletePtrArr(pWinFlagAry);
	}

    return TRUE;
}

BOOL CFOPGDIPlusProxy::DrawPolyPolygonBezier(CDC *pDC,const CRect &rcPos, ULONG nPoly, const ULONG* pPoints,
											const fopPointExt* const* pPtAry, const BYTE* const* pFlgAry,
											COLORREF crStart,COLORREF crEnd,UINT nType, CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DrawPolyPolygonBezier(pDC, rcPos, nPoly,pPoints, pPtAry, pFlgAry, 
			crStart, crEnd, nType, pBrush);
		return TRUE;
	}

	if(nType >= 92)
	{
	//	if(nType != FOP_CUSTOM_GRADIENT_TYPE)
			CFOPVisualProxy::DrawPolyPolygonBezier(pDC, rcPos, nPoly,pPoints, pPtAry, pFlgAry, 
			crStart, crEnd, nType, pBrush);

		ULONG nCurrPoly, nTotalPoints;
		const ULONG* pCurrPoints = pPoints;
		for( nCurrPoly=0, nTotalPoints=0; nCurrPoly<nPoly; ++nCurrPoly )
			nTotalPoints += *pCurrPoints++;
		
		POINT	aStackAry1[FOP_PLOY_STACKBUF];
		BYTE	aStackAry2[FOP_PLOY_STACKBUF];
		POINT* 	pWinPointAry;
		BYTE* 	pWinFlagAry;
		if( nTotalPoints > FOP_PLOY_STACKBUF )
		{
			pWinPointAry = new POINT[ nTotalPoints ];
			pWinFlagAry = new BYTE[ nTotalPoints ];
		}
		else
		{
			pWinPointAry = aStackAry1;
			pWinFlagAry = aStackAry2;
		}
		
		FOPPreparePolyDrawG(TRUE, nPoly, pPoints, pPtAry, pFlgAry, pWinPointAry, pWinFlagAry);
		
		FOPPolyDrawG(pDC,m_nTransparent, pWinPointAry, pWinFlagAry, nTotalPoints, FALSE, crStart, crEnd, nType);
		
		if( pWinPointAry != aStackAry1 )
		{
			_SafeDeletePtrArr(pWinPointAry);
			_SafeDeletePtrArr(pWinFlagAry);
		}
		return TRUE;
	}

    ULONG nCurrPoly, nTotalPoints;
    const ULONG* pCurrPoints = pPoints;
    for( nCurrPoly=0, nTotalPoints=0; nCurrPoly<nPoly; ++nCurrPoly )
        nTotalPoints += *pCurrPoints++;

    POINT	aStackAry1[FOP_PLOY_STACKBUF];
    BYTE	aStackAry2[FOP_PLOY_STACKBUF];
    POINT* 	pWinPointAry;
    BYTE* 	pWinFlagAry;
    if( nTotalPoints > FOP_PLOY_STACKBUF )
    {
        pWinPointAry = new POINT[ nTotalPoints ];
        pWinFlagAry = new BYTE[ nTotalPoints ];
    }
    else
    {
        pWinPointAry = aStackAry1;
        pWinFlagAry = aStackAry2;
    }

    FOPPreparePolyDrawG(TRUE, nPoly, pPoints, pPtAry, pFlgAry, pWinPointAry, pWinFlagAry);

	FOPPolyDrawG(pDC,m_nTransparent, pWinPointAry, pWinFlagAry, nTotalPoints, TRUE, crStart, crEnd, nType);

    if( pWinPointAry != aStackAry1 )
    {
		_SafeDeletePtrArr(pWinPointAry);
		_SafeDeletePtrArr(pWinFlagAry);
    }
    return TRUE;
}

#define FOP_POLYPOLY_STACKBUF		32
 
BOOL CFOPGDIPlusProxy::FillNewPolyPolygon(CDC *pDC,const CRect &rcPos, USHORT nPoly, 
										 const FOPSimpleCompositePolygon& rPolyPoly,
										 COLORREF crStart,COLORREF crEnd,UINT nFillType,
										 CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillNewPolyPolygon(pDC, rcPos, nPoly,rPolyPoly,crStart, crEnd, nFillType, pBrush);
		return TRUE;
	}

	BOOL bReturn		= TRUE;
    ULONG				aStackAry1[FOP_POLYPOLY_STACKBUF];
    FOPCONSTPOINT		aStackAry2[FOP_POLYPOLY_STACKBUF];
    BYTE*				aStackAry3[FOP_POLYPOLY_STACKBUF];
    ULONG*				pPointAry;
    FOPCONSTPOINT* 		pPointAryAry;
    const BYTE**	 	pFlagAryAry;
    USHORT				i = 0, j = 0, last = 0;
    BOOL				bHaveBezier = FALSE;

    if ( nPoly > FOP_POLYPOLY_STACKBUF )
    {
        pPointAry		= new ULONG[nPoly];
        pPointAryAry	= new FOPCONSTPOINT[nPoly];
        pFlagAryAry		= new const BYTE*[nPoly];
    }
    else
    {
        pPointAry		= aStackAry1;
        pPointAryAry	= aStackAry2;
        pFlagAryAry		= (const BYTE**)aStackAry3;
    }

    do
    {
        const FOPSimplePolygon&	rPoly = rPolyPoly[ i ];
        int nSize = rPoly.GetPointCount();
        if ( nSize )
        {
            pPointAry[j]	= nSize;
            pPointAryAry[j] = (FOPCONSTPOINT )rPoly.GetConstPointAry();
            pFlagAryAry[j]  = rPoly.GetConstFlagAry();
            last 			= i;

            if( pFlagAryAry[j] )
			{
                bHaveBezier = TRUE;
			}

            ++j;
        }

        ++i;
    }
    while ( i < nPoly );

    if ( j == 1 )
    {
        if( bHaveBezier )
		{
			FillNewPolygonBezier(pDC,rcPos, *pPointAry, *pPointAryAry, *pFlagAryAry,crStart,crEnd,nFillType,pBrush);
        }
        else
        {
			FillPoly(pDC,(POINT *)*pPointAryAry,*pPointAry,crStart,crEnd,nFillType,pBrush);
        }
    }
    else
    {
        if( bHaveBezier )
        {
            DrawPolyPolygonBezier(pDC,rcPos,j,pPointAry, pPointAryAry, pFlagAryAry,crStart,crEnd,nFillType,pBrush);
        }
        else
        {
			bReturn = FALSE;
        }
    }

    if ( pPointAry != aStackAry1 )
    {
        _SafeDeletePtrArr(pPointAry);
        _SafeDeletePtrArr(pPointAryAry);
        _SafeDeletePtrArr(pFlagAryAry);
    }

	//m_nTransparent = 255;

	return bReturn;
}

void CFOPGDIPlusProxy::DoDrawArrowLine(CDC *pDC,const CPoint &ptStart,const CPoint &ptEnd)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DoDrawArrowLine(pDC, ptStart, ptEnd);
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	if (NULL != pPenG)
		graphics.DrawLine(pPenG, Point(ptStart.x, ptStart.y), Point(ptEnd.x, ptEnd.y));
 
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::DoDrawArrowAngleArcLine(CDC *pDC,const CPoint &ptStart,const CPoint &ptEnd,
		const int &nWidth,const float &startAngle,const float &endAngle,const COLORREF &crFill,const BOOL &bFill)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DoDrawArrowAngleArcLine(pDC, ptStart, ptEnd, nWidth, startAngle, endAngle, crFill, bFill);
		return;
	}

	if(m_nTransparent == 255)
	{
		CFOPVisualProxy::DoDrawArrowAngleArcLine(pDC, ptStart, ptEnd, nWidth, startAngle, endAngle, crFill, bFill);
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		if (NULL != pPenG)
			graphics.DrawArc(pPenG, ptEnd.x-nWidth, ptEnd.y-nWidth, 2*nWidth, 2*nWidth,startAngle,endAngle);
		
		
		//m_nTransparent = 255;
	 
		_SafeDeletePtr(pPenG);
		return;
	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	if (NULL != pPenG)
		graphics.DrawArc(pPenG, ptEnd.x-nWidth, ptEnd.y-nWidth, 2*nWidth, 2*nWidth,startAngle,endAngle);

	if (bFill)
	{
		Brush* pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(1, crFill,RGB(0,0,0), m_nTransparent, m_nTransparent);
		if (NULL != pBrushG)
			graphics.FillEllipse(pBrushG, ptEnd.x - nWidth, ptEnd.y - nWidth, 2*nWidth, 2*nWidth);

		_SafeDeletePtr(pBrushG);
	}
	
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::DrawText(CDC *pDC,const CString &strText, CRect rcRect, UINT uFormat)
{
// 	if(gfxData.m_bIsHMIMode)
// 	{
// 		CFOPVisualProxy::DrawText(pDC, strText, rcRect, uFormat);
// 			return;
// 	}

// 	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
// 	{
// 		if(gfxData.m_bWithGDIPlusText)
// 		{
// 			CFOPVisualProxy::DrawText(pDC, strText, rcRect, uFormat);
// 			return;
// 		}
// 	}

	if(strText.IsEmpty())
	{
		return;
	}

	int nLength = 0;
	
	if(nLength <= 0)
	{
		nLength = strText.GetLength();
	}
	
	Gdiplus::Font* pFontG = CFOPHokFontCache::GetInstance()->GetFont(pDC->GetCurrentFont(), pDC);
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	CFont *pFont = pDC->GetCurrentFont();
	
	LOGFONT lf;
	ZeroMemory(&lf, sizeof(lf));
	pFont->GetLogFont(&lf);
	
	CPoint ptText = rcRect.CenterPoint();
	if (NULL != pFontG)
	{
	//	UINT uFormat = DT_LEFT|DT_TOP|DT_SINGLELINE;
	
		COLORREF crText = pDC->GetTextColor();
		Brush *pBrushG = NULL;
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(1, crText, crText, m_nTransparent, m_nTransparent);
		StringFormat stringFormat(StringFormat::GenericTypographic());
		stringFormat.SetLineAlignment(uFormat & DT_VCENTER ? StringAlignmentCenter: StringAlignmentNear);
		stringFormat.SetAlignment(uFormat & DT_CENTER ? StringAlignmentCenter : uFormat & DT_RIGHT ? StringAlignmentFar : StringAlignmentNear);
		
		graphics.TranslateTransform(ptText.x, ptText.y);
		graphics.RotateTransform(-lf.lfOrientation/10);
		graphics.TranslateTransform(-ptText.x, -ptText.y);
		
		RectF rectx((REAL)rcRect.left, (REAL)rcRect.top, (REAL)rcRect.right - rcRect.left, (REAL)rcRect.bottom - rcRect.top);

		FOP_DrawString(&graphics, strText, *pFontG, *pBrushG, rectx, stringFormat);

		_SafeDeletePtr(pBrushG);
	//	_SafeDeletePtr(pFontG);
		//		_SafeDeletePtr(pBrushG);
	}
	
}

void CFOPGDIPlusProxy::S_FillPolygon(CDC *pDC, POINT* points, int nCount,const COLORREF &crFill, const BOOL &bNullBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::S_FillPolygon(pDC, points, nCount, crFill);
		return;
	}

	if(0)//m_nTransparent == 255)
	{
		CFOPVisualProxy::S_FillPolygon(pDC, points, nCount, crFill);
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		if (NULL != pPenG)
			graphics.DrawPolygon(pPenG, (const Point *)points, nCount);
		
		//m_nTransparent = 255;
		_SafeDeletePtr(pPenG);
		return;
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	Brush* pBrushG = NULL;
	if(!bNullBrush)
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(1, crFill,RGB(0,0,0), m_nTransparent, m_nTransparent);
	
	if (NULL != pPenG)
		graphics.DrawPolygon(pPenG, (const Point *)points, nCount);
	if (NULL != pBrushG)
		graphics.FillPolygon(pBrushG, (const Point *)points, nCount);
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}


void CFOPGDIPlusProxy::S_Ellipse(CDC *pDC, LPCRECT lpRect,const COLORREF &crFill, const BOOL &bNullBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::S_Ellipse(pDC, lpRect, crFill);
		return;
	}

	if(0)//m_nTransparent == 255)
	{
		CFOPVisualProxy::S_Ellipse(pDC, lpRect, crFill);
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		if (NULL != pPenG)
			graphics.DrawEllipse(pPenG, lpRect->left, lpRect->top, lpRect->right - lpRect->left, lpRect->bottom - lpRect->top);
		
		//m_nTransparent = 255;
		
		return;
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	Brush* pBrushG = NULL;
	if(!bNullBrush)
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(1, crFill,RGB(0,0,0), m_nTransparent, m_nTransparent);
	
	if (NULL != pPenG)
		graphics.DrawEllipse(pPenG, lpRect->left, lpRect->top, lpRect->right - lpRect->left, lpRect->bottom - lpRect->top);
	if (NULL != pBrushG)
		graphics.FillEllipse(pBrushG, lpRect->left, lpRect->top, lpRect->right - lpRect->left, lpRect->bottom - lpRect->top);
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::DoFillComplexGeometry(CDC *pDC, const FOPComplexGeometry &geometry, 
											COLORREF crStart,COLORREF crEnd,UINT nFillType,CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DoFillComplexGeometry(pDC, geometry, crStart, crEnd, nFillType, pBrush);
		return;
	}
	
	Gdiplus::GraphicsPath* pGeometry = X_FGenPath(geometry);
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	
	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = geometry.rcBound;
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart,RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, pGeometry);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd,RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, pGeometry);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPath(pBrushG, pGeometry);
	}
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, pGeometry);
	
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);

	_SafeDeletePtr(pGeometry);
}	

void CFOPGDIPlusProxy::S_RoundRect(CDC *pDC, LPCRECT lpRect,CPoint ptCorner, const COLORREF &crFill, const BOOL &bNullBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::S_RoundRect(pDC, lpRect, ptCorner, crFill);
		return;
	}
	
	if(0)//m_nTransparent == 255)
	{
		CFOPVisualProxy::S_RoundRect(pDC, lpRect,ptCorner, crFill);
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		if (NULL != pPenG)
			graphics.DrawRectangle(pPenG, lpRect->left, lpRect->top, lpRect->right - lpRect->left, lpRect->bottom - lpRect->top);
		
		//m_nTransparent = 255;
		
		return;
	}
	FOPRect theRect(lpRect);

	REAL X1 = theRect.left;
	REAL X2 = theRect.left+theRect.Width();
	REAL Y1 = theRect.top;
	REAL Y2 = theRect.top+theRect.Height();
	REAL W =  ptCorner.x/2;
	REAL H =  ptCorner.y/2;
	
	if (2*W >= theRect.Width())
		W = theRect.Width()/2;
	
	if (2*H >= theRect.Height())
		H = theRect.Height()/2;
	
	GraphicsPath m_Path;
	m_Path.AddArc(X1,Y1,2*W,2*H,180,90);
	m_Path.AddArc(X2-2*W,Y1,2*W,2*H,270,90);
	m_Path.AddArc(X2-2*W,Y2-2*H,2*W,2*H,0,90);
	m_Path.AddArc(X1,Y2-2*H,2*W,2*H,90,90);
	m_Path.CloseFigure();
	
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);

	Brush* pBrushG = NULL;
	if(!bNullBrush)
	    pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(1, crFill,RGB(0,0,0), m_nTransparent, m_nTransparent);
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &m_Path);
	if (NULL != pBrushG)
		graphics.FillPath(pBrushG, &m_Path);
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::S_Rect(CDC *pDC, LPCRECT lpRect,const COLORREF &crFill, const BOOL &bNullBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::S_Rect(pDC, lpRect, crFill);
		return;
	}
	
	if(0)//m_nTransparent == 255)
	{
		CFOPVisualProxy::S_Rect(pDC, lpRect, crFill);
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
		
		if (NULL != pPenG)
			graphics.DrawRectangle(pPenG, lpRect->left, lpRect->top, lpRect->right - lpRect->left, lpRect->bottom - lpRect->top);
		
		//m_nTransparent = 255;
		
		return;
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	Brush* pBrushG = NULL;
	if(!bNullBrush)
	    pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(1, crFill,RGB(0,0,0), m_nTransparent, m_nTransparent);
	
	if (NULL != pPenG)
		graphics.DrawRectangle(pPenG, lpRect->left, lpRect->top, lpRect->right - lpRect->left, lpRect->bottom - lpRect->top);
	if (NULL != pBrushG)
		graphics.FillRectangle(pBrushG, lpRect->left, lpRect->top, lpRect->right - lpRect->left, lpRect->bottom - lpRect->top);
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}


void CFOPGDIPlusProxy::GenSplinePoints(LPPOINT m_lpShapePoints, int m_nCompPtCount, 
									  LPPOINT& lpSplinePoints, int& nSpPtCount)
{
	GraphicsPath path;
	LPPOINT m_points = new POINT[m_nCompPtCount];
	::memcpy(m_points, m_lpShapePoints, sizeof(POINT)*m_nCompPtCount);
	int m_nPoint = m_nCompPtCount;
	int alloclen = m_nPoint+ 1;
    PointF *points = (PointF *) _alloca (alloclen * sizeof(PointF)); 
	
	for (int i=0;i<m_nPoint;i++)
	{
		points[i].X = (float)m_points[i].x;
		points[i].Y = (float)m_points[i].y;
	}
	
	points[m_nPoint].X = (float)m_points[0].x;
	points[m_nPoint].Y = (float)m_points[0].y;
	
	if (m_nPoint <=2) 
	{
		path.AddLines(points,m_nPoint);
	}
	else 
	{
		path.AddClosedCurve(points,m_nPoint, 0.5);
		
	}

	int nCount = path.GetPointCount();
	// 	
	PointF* lppt = new PointF[nCount];
	path.GetPathPoints(lppt, nCount);
	
	POINT *ltemp = new POINT[nCount];
	int x = 0; 
	for(x = 0; x < nCount; x++)
	{ 
		ltemp[x].x = (long)lppt[x].X;
		ltemp[x].y = (long)lppt[x].Y;
	}
	
	// 	
	BYTE* pTypes = new BYTE[nCount];
	path.GetPathTypes(pTypes, nCount);
	
	FOPSimplePolygon poly;
	poly.ParsePath(ltemp, pTypes, nCount, TRUE);


	FOPPolygon polyTemp = FOPCreatePolygon(poly,NULL);
	
	CPoint* pPoint = (CPoint*) polyTemp.GetConstPointAry();
	nSpPtCount = polyTemp.GetSize();

	lpSplinePoints = new POINT[nSpPtCount];

	x = 0;
	for(x = 0; x < nSpPtCount; x++)
	{
		lpSplinePoints[x] = pPoint[x];
	}

	delete [] ltemp;
	delete [] lppt;
	delete [] pTypes;

	delete [] m_points;
	m_points = NULL;
}


void CFOPGDIPlusProxy::DoDrawExtTrackCurve(CDC *pDC, POINT* m_lpShapePoints, int m_nCompPtCount)
{
	GraphicsPath path;
	LPPOINT m_points = new POINT[m_nCompPtCount];
	::memcpy(m_points, m_lpShapePoints, sizeof(POINT)*m_nCompPtCount);
	int m_nPoint = m_nCompPtCount;
	int alloclen = m_nPoint+ 1;
    PointF *points = (PointF *) _alloca (alloclen * sizeof(PointF)); 
	
	for (int i=0;i<m_nPoint;i++)
	{
		points[i].X = (float)m_points[i].x;
		points[i].Y = (float)m_points[i].y;
	}
	
	points[m_nPoint].X = (float)m_points[0].x;
	points[m_nPoint].Y = (float)m_points[0].y;
	
	if (m_nPoint <=2) 
	{
		path.AddLines(points,m_nPoint);
	}
	else 
	{
		path.AddCurve(points,m_nPoint, 0.5);
		
	}
	
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &path);
	
	//m_nTransparent = 255;

	delete [] m_points;
	m_points = NULL;
 
	_SafeDeletePtr(pPenG);
}


void CFOPGDIPlusProxy::DoDrawExtCloseCurve(CDC *pDC, POINT* m_lpShapePoints, int m_nCompPtCount, const CRect &rcPos,COLORREF crStart,
										  COLORREF crEnd,UINT nFillType,CBrush *pBrush)
{
	pBrush;
	GraphicsPath path;
	LPPOINT m_points = new POINT[m_nCompPtCount];
	::memcpy(m_points, m_lpShapePoints, sizeof(POINT)*m_nCompPtCount);
	int m_nPoint = m_nCompPtCount;
	int alloclen = m_nPoint+ 1;
    PointF *points = (PointF *) _alloca (alloclen * sizeof(PointF)); 
	
	for (int i=0;i<m_nPoint;i++)
	{
		points[i].X = (float)m_points[i].x;
		points[i].Y = (float)m_points[i].y;
	}
	
	points[m_nPoint].X = (float)m_points[0].x;
	points[m_nPoint].Y = (float)m_points[0].y;
	
	if (m_nPoint <=2) 
	{
		path.AddLines(points,m_nPoint);
	}
	else 
	{
		path.AddClosedCurve(points,m_nPoint, 0.5);

	}

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	
	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = rcPos;
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &path);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &path);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPath(pBrushG, &path);
	}
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &path);
	
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;

	delete [] m_points;
	m_points = NULL;
}


void CFOPGDIPlusProxy::S_DrawCurve(CDC *pDC, POINT* lppt, BYTE* pTypes, int nCount, const BOOL &bClose)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::S_DrawCurve(pDC, lppt, pTypes, nCount, bClose);
		return;
	}
	
	GraphicsPath path((Point* )lppt, pTypes, nCount);

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &path);
	 
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

void CFOPGDIPlusProxy::DoDrawArrowPolygon(CDC *pDC,LPPOINT points,int nCount,const COLORREF &crFill)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::DoDrawArrowPolygon(pDC, points, nCount, crFill);
		return;
	}

	if(m_nTransparent == 255)
	{
		CFOPVisualProxy::DoDrawArrowPolygon(pDC, points, nCount, crFill);
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
			
		if (NULL != pPenG)
			graphics.DrawPolygon(pPenG, (const Point *)points, nCount);
		
		//m_nTransparent = 255;
		 
		_SafeDeletePtr(pPenG);
		return;
	}

	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	Brush* pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(1, crFill,RGB(0,0,0), m_nTransparent, m_nTransparent);

	if (NULL != pPenG)
		graphics.DrawPolygon(pPenG, (const Point *)points, nCount);
	if (NULL != pBrushG)
		graphics.FillPolygon(pBrushG, (const Point *)points, nCount);

	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
	//m_nTransparent = 255;
}

BOOL CFOPGDIPlusProxy::IsGDIPlus()
{
	return TRUE;
}


void CFOPGDIPlusProxy::FillEllipse(CDC *pDC, const FOPRect &rect, COLORREF crStart,COLORREF crEnd,
								  UINT nFillType,CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillEllipse(pDC, rect,crStart,crEnd,nFillType,pBrush);
		return;
	}

	if(nFillType >= 92)
	{
		if(1)//if(nFillType != FOP_CUSTOM_GRADIENT_TYPE)
		CFOPVisualProxy::FillEllipse(pDC, rect,crStart,crEnd,nFillType,pBrush);
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),255);
		
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
		{
		
			RectF r(rect.left, rect.top, rect.Width(), rect.Height());
			Brush *pBrSolid = NULL;
			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
				pBrSolid = CreatePathBrush(r);
			else
				pBrSolid = CreateRadialBrush(r);
			
			if(pBrSolid != NULL)
				graphics.FillEllipse(pBrSolid,rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
			_SafeDeletePtr(pBrSolid);
		}
		if (NULL != pPenG)
			graphics.DrawEllipse(pPenG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
		
	 
		_SafeDeletePtr(pPenG);
		//m_nTransparent = 255;
		return;
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	int nTransparent = m_nTransparent;
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(), nTransparent);
	if (0 == nTransparent)
	{
		if (NULL != pPenG)
		{
			graphics.DrawEllipse(pPenG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
		}
		return;
	}
	
	Brush *pBrushG = NULL;
	if (nTransparent != 0)
	{		
		if (41 < nFillType && nFillType < 62)
		{
			CRect rc = rect;
			pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
		}
		else
		{
			pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
		}
	}
	
	if (NULL != pBrushG)
	{

		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillEllipse(pBrSolid, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillEllipse(pBrSolid, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillEllipse(pBrushG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);

	}
	
	if (NULL != pPenG)
	{
		graphics.DrawEllipse(pPenG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
	}	
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
}

Brush*	CFOPGDIPlusProxy::CreatePathBrush(RectF rect)
{
	int nCount = GetGradient()->GetStops()->GetSize();
	if(nCount < 2)
	{
		return NULL;
	}

	int nAngle = GetGradient()->GetAngle();
	int nMode = 0;
	if(nAngle == 90 || nAngle == 270 || nAngle == -90)
	{
		nMode = 1;
	}
	else if(nAngle != 0)
	{
		nMode = 2;
	}
	int index = 0;
	int numElements;
	int r, g, b, a;
	COLORREF color;
	FO_COLORSTOP stop( GetGradient()->GetStops()->GetAt(0) );
	numElements = nCount;
	Color *colors = new Color[numElements];
	REAL  *positions = new REAL[numElements];
	stop = GetGradient()->GetStops()->GetAt(0);
	color = stop.crStop;
	colors[0] = Color(stop.fOpacity *255, GetRValue(color), GetGValue(color), GetBValue(color));
	positions[0] = 0.0;
	
	stop = GetGradient()->GetStops()->GetAt(nCount - 1);
	color = stop.crStop;
	colors[numElements - 1] = Color(stop.fOpacity *255, GetRValue(color), GetGValue(color), GetBValue(color));
	positions[numElements - 1] = 1.0;
	
	for( int xindex = 0 ; xindex < nCount; xindex++)
	{
		stop = GetGradient()->GetStops()->GetAt(xindex);
		color = stop.crStop;
		r = GetRValue(color);
		g = GetGValue(color);
		b = GetBValue(color);
		a = stop.fOpacity * 255;
		colors[xindex] = Color(a, r, g, b);
		positions[xindex] = stop.fOffset;
	}

	LinearGradientBrush* linGrBrush = NULL;
	if(nMode == 1)
		linGrBrush = new LinearGradientBrush ( rect,colors[0], colors[numElements -1], LinearGradientModeHorizontal);
	else if(nMode == 0)
		linGrBrush = new LinearGradientBrush ( rect, colors[0], colors[numElements -1], LinearGradientModeVertical);
	else
		linGrBrush = new LinearGradientBrush ( PointF(rect.X, rect.Y), PointF(rect.GetRight(), rect.GetBottom()), colors[0], colors[numElements -1]);

	linGrBrush->SetInterpolationColors(colors, positions, numElements);
	delete[] colors;
	delete[] positions;

	return linGrBrush;
}

Brush*	CFOPGDIPlusProxy::CreateRadialBrush(RectF rect)
{
	int nCount = GetGradient()->GetStops()->GetSize();
	if(nCount < 2)
	{
		return NULL;
	}
	
	int nAngle = GetGradient()->GetAngle();
	int nMode = 0;
	if(nAngle == 90 || nAngle == 270 || nAngle == -90)
	{
		nMode = 1;
	}
	else if(nAngle != 0)
	{
		nMode = 2;
	}
	int index = 0;
	int numElements;
	int r, g, b, a;
	COLORREF color;
	FO_COLORSTOP stop( GetGradient()->GetStops()->GetAt(0) );
	numElements = nCount;
	Color *colors = new Color[numElements];
	REAL  *positions = new REAL[numElements];
	stop = GetGradient()->GetStops()->GetAt(0);
	color = stop.crStop;
	colors[0] = Color(stop.fOpacity *255, GetRValue(color), GetGValue(color), GetBValue(color));
	positions[0] = 0.0;
	
	stop = GetGradient()->GetStops()->GetAt(nCount - 1);
	color = stop.crStop;
	colors[numElements - 1] = Color(stop.fOpacity *255, GetRValue(color), GetGValue(color), GetBValue(color));
	positions[numElements - 1] = 1.0;
	
	for( int xindex = 0 ; xindex < nCount; xindex++)
	{
		stop = GetGradient()->GetStops()->GetAt(xindex);
		color = stop.crStop;
		r = GetRValue(color);
		g = GetGValue(color);
		b = GetBValue(color);
		a = stop.fOpacity * 255;
		colors[xindex] = Color(a, r, g, b);
		positions[xindex] = stop.fOffset;
	}
	GraphicsPath gradientPath;
	gradientPath.AddEllipse(rect.X, rect.Y, rect.Width, rect.Height);

	PathGradientBrush* linGrBrush = NULL;
	linGrBrush = new PathGradientBrush (&gradientPath);
	linGrBrush->SetCenterPoint( PointF(rect.X + rect.Width/2, rect.Y + rect.Height/2) );
	linGrBrush->SetCenterColor( colors[numElements - 1] );
	linGrBrush->SetInterpolationColors( colors, positions, numElements );

	delete[] colors;
	delete[] positions;
	
	return linGrBrush;
}

void CFOPGDIPlusProxy::FillRect(CDC *pDC, FOPRect rect, COLORREF crStart,
							   COLORREF crEnd,UINT nFillType,CBrush *pBrush)
{
	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillRect(pDC, rect,crStart,crEnd,nFillType,pBrush);
		return;
	}
	
	if(nFillType >= 92)
	{
		if(1)//if(nFillType != FOP_CUSTOM_GRADIENT_TYPE)
			CFOPVisualProxy::FillRect(pDC, rect,crStart,crEnd,nFillType,pBrush);
		
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),255);
		
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);

		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
		{
			RectF r(rect.left, rect.top, rect.Width(), rect.Height());
			Brush *pBrSolid = NULL;
			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
				pBrSolid = CreatePathBrush(r);
			else
				pBrSolid = CreateRadialBrush(r);
			
			if(pBrSolid != NULL)
				graphics.FillRectangle(pBrSolid, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
			_SafeDeletePtr(pBrSolid);
		}
		
		if (NULL != pPenG)
			graphics.DrawRectangle(pPenG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
		GetGradient()->GetStops()->RemoveAll();
 
		_SafeDeletePtr(pPenG);
		//m_nTransparent = 255;
		return;
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	int nTransparent = m_nTransparent;
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(), nTransparent);
	if (0 == nTransparent)
	{
		if (NULL != pPenG)
		{
			graphics.DrawRectangle(pPenG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
		}
		return;
	}
	
	Brush *pBrushG = NULL;
	if (nTransparent != 0)
	{		
		if (41 < nFillType && nFillType < 62)
		{
			CRect rc = rect;
			pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
		}
		else
		{
			pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
		}
	}
	
	if (NULL != pBrushG)
	{
		
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillRectangle(pBrSolid, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillRectangle(pBrSolid, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillRectangle(pBrushG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
		
	}
	
	if (NULL != pPenG)
	{
		graphics.DrawRectangle(pPenG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
	}	
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
}

void CFOPGDIPlusProxy::FillRoundRect(CDC *pDC, const FOPRect &theRect, const FOPPoint &m_roundness,
									COLORREF crStart,COLORREF crEnd,UINT nFillType,CBrush *pBrush)
{

	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
	{
		CFOPVisualProxy::FillRoundRect(pDC, theRect, m_roundness, crStart, crEnd, nFillType, pBrush);
		return;
	}
	
	REAL X1 = theRect.left;
	REAL X2 = theRect.left+theRect.Width();
	REAL Y1 = theRect.top;
	REAL Y2 = theRect.top+theRect.Height();
	REAL W =  m_roundness.x/2;
	REAL H =  m_roundness.y/2;
	
	if (2*W >= theRect.Width())
		W = theRect.Width()/2;
	
	if (2*H >= theRect.Height())
		H = theRect.Height()/2;
	
	GraphicsPath m_Path;
	m_Path.AddArc(X1,Y1,2*W,2*H,180,90);
	m_Path.AddArc(X2-2*W,Y1,2*W,2*H,270,90);
	m_Path.AddArc(X2-2*W,Y2-2*H,2*W,2*H,0,90);
	m_Path.AddArc(X1,Y2-2*H,2*W,2*H,90,90);
	m_Path.CloseFigure();
	

	if(nFillType >= 92)
	{
		if(1)//if(nFillType != FOP_CUSTOM_GRADIENT_TYPE)
		CFOPVisualProxy::FillRoundRect(pDC, theRect, m_roundness, crStart, crEnd, nFillType, pBrush);
		Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),255);
		
		Graphics graphics(pDC->m_hDC);
		PrepareGraphics(&graphics);
	
		if(0)//if(nFillType == FOP_CUSTOM_GRADIENT_TYPE)
		{
		
			RectF r(theRect.left, theRect.top, theRect.Width(), theRect.Height());
			Brush *pBrSolid = NULL;
			if(GetGradient()->GetType() == FOP_GRADIENT_LINEAR)
				pBrSolid = CreatePathBrush(r);
			else
				pBrSolid = CreateRadialBrush(r);
			
			if(pBrSolid != NULL)
				graphics.FillPath(pBrSolid, &m_Path);
			_SafeDeletePtr(pBrSolid);
		}
		if (NULL != pPenG)
			graphics.DrawPath(pPenG, &m_Path);
		
		 
		_SafeDeletePtr(pPenG);
		//m_nTransparent = 255;
		return;
	}
	
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(),m_nTransparent);
	
	Brush *pBrushG = NULL;
	if ( 41 < nFillType && nFillType <62  )
	{
		CRect rc = theRect;
		pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, m_nTransparent, m_nTransparent);
	}
	else
	{
		pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, m_nTransparent, m_nTransparent);
	}
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);

	if (NULL != pBrushG)
	{
		if (nFillType-42 == GRADIENT_ELLIPSE_OUT)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crStart, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &m_Path);
			_SafeDeletePtr(pBrSolid);
		}
		if (nFillType-42 == GRADIENT_ELLIPSE_IN)
		{
			Brush *pBrSolid = CFOPHokBrushCache::GetInstance()->GetBrush(1, crEnd, RGB(0,0,0), m_nTransparent, m_nTransparent);
			graphics.FillPath(pBrSolid, &m_Path);
			_SafeDeletePtr(pBrSolid);
		}
		graphics.FillPath(pBrushG, &m_Path);
	}
	
	if (NULL != pPenG)
		graphics.DrawPath(pPenG, &m_Path);
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);

}

BOOL CFOPGDIPlusProxy::FillCircle(CDC *pDC,
								  const CRect &rcPos, 
								  COLORREF crStart,
								  COLORREF crEnd,
								  UINT nType, 
								  CBrush *pBrush)
{
	FillEllipse(pDC, rcPos, crStart, crEnd, nType, pBrush);
	return TRUE;
}

void CFOPGDIPlusProxy::FODrawCircle(CDC *pDC, 
								  const BYTE &nTrans, 
								  const CRect& rect,
								  BOOL bFill, 
								  COLORREF crStart, 
								  COLORREF crEnd, 
								  UINT nFillType)
{
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	int nTransparent = m_nTransparent;
	Pen* pPenG = CFOPHokPenCache::GetInstance()->GetPen(pDC->GetCurrentPen(), nTransparent);
	if (0 == nTransparent)
	{
		if (NULL != pPenG)
		{
			graphics.DrawEllipse(pPenG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
		}
 
		_SafeDeletePtr(pPenG);
		return;
	}
	
	Brush *pBrushG = NULL;
	if (bFill == TRUE && nTransparent != 0)
	{		
		if (41 < nFillType && nFillType < 62)
		{
			CRect rc = rect;
			pBrushG = CFOPHokBrushCache::CreateGradientBrush(nFillType, crStart, crEnd, rc, nTrans, nTrans);
		}
		else
		{
			pBrushG = CFOPHokBrushCache::GetInstance()->GetBrush(nFillType, crStart, crEnd, nTrans, nTrans);
		}
	}
	
	if (NULL != pBrushG)
	{
		graphics.FillEllipse(pBrushG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
	}
	
	if (NULL != pPenG)
	{
		graphics.DrawEllipse(pPenG, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
	}	
	
	_SafeDeletePtr(pBrushG);
	_SafeDeletePtr(pPenG);
}

// Drawing text helper string.
#pragma warning( disable : 4244 )

CFOPWideCharString::CFOPWideCharString()
{
	m_nBufferLen=0;
	m_pBuffer=new WCHAR [m_nBufferLen+1];
	m_pBuffer[0]=0;
}

CFOPWideCharString::CFOPWideCharString(LPCWSTR widestring, int len)
{
	if(widestring)
		m_nBufferLen=wcslen(widestring);
	else
		m_nBufferLen=0;
	if(len<m_nBufferLen && len>=0){
		m_nBufferLen=len;
	}

	m_pBuffer=new WCHAR [m_nBufferLen+1];
	//if(widestring)
	//	wcsncpy(m_pBuffer, widestring,m_nBufferLen);
	m_pBuffer[m_nBufferLen]=0;
}

CFOPWideCharString::CFOPWideCharString(const CFOPWideCharString& wcString)
{
	m_nBufferLen=wcString.m_nBufferLen;
	m_pBuffer=new WCHAR [m_nBufferLen+1];
//	wcscpy(m_pBuffer,wcString.m_pBuffer);
}

CFOPWideCharString::CFOPWideCharString(const char* lpszString, int len)
{
	if(!lpszString){
		m_nBufferLen=0;
		m_pBuffer=new WCHAR [1];
		m_pBuffer[0]=0;
		return;
	}
	int srclen=strlen(lpszString);

	if(len < srclen && len >=0){
		srclen=len;
	}

	m_nBufferLen=srclen;
	m_pBuffer=new WCHAR[m_nBufferLen+1];
	m_nBufferLen=MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,lpszString,srclen,m_pBuffer,m_nBufferLen+1);
	m_pBuffer[m_nBufferLen]=0;
}

CFOPWideCharString::~CFOPWideCharString()
{
	if(m_pBuffer)
		delete[] m_pBuffer;
}

CFOPWideCharString& CFOPWideCharString::operator=(const CFOPWideCharString& wcString)
{
	if(this == &wcString)
		return *this;

	if(m_pBuffer)
		delete[] m_pBuffer;
	m_nBufferLen=wcString.m_nBufferLen;
	m_pBuffer=new WCHAR [m_nBufferLen+1];
//	wcscpy(m_pBuffer,wcString.m_pBuffer);
	return *this;
}

CFOPWideCharString& CFOPWideCharString::operator =(const char* lpszString)
{
	if(lpszString==NULL){
		m_nBufferLen=0;
		if(!m_pBuffer){
			m_pBuffer=new WCHAR [m_nBufferLen+1];
		}
		m_pBuffer[0]=0;
		return *this;
	}
	
	int srclen=strlen(lpszString);
	m_nBufferLen=srclen;
	if(m_pBuffer)
		delete[] m_pBuffer;
	m_pBuffer=new WCHAR[m_nBufferLen+1];
	m_nBufferLen=MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,lpszString,srclen,m_pBuffer,m_nBufferLen+1);
	m_pBuffer[m_nBufferLen]=0;
	return *this;	

}

CFOPWideCharString CFOPWideCharString::Mid(int begin,int len)
{
	if (len<0) len=GetLength();
	if(begin+len>m_nBufferLen)
		len=m_nBufferLen-begin;

	return CFOPWideCharString(m_pBuffer+begin,len);
}
BOOL CFOPWideCharString::operator == (const CFOPWideCharString& wcString)
{
	return (wcscmp(m_pBuffer,wcString.m_pBuffer)==0);
}
BOOL CFOPWideCharString::operator != (const CFOPWideCharString& wcString)
{
	return (wcscmp(m_pBuffer,wcString.m_pBuffer)!=0);
}
WCHAR CFOPWideCharString::operator[] (int index) const
{
	if(index<0 || index>=m_nBufferLen)
		return 0;
	return m_pBuffer[index];
}

WCHAR& CFOPWideCharString::operator[] (int index)
{
	if(index<0 || index>=m_nBufferLen){
		static WCHAR w=0;
		return w;
	}
	return m_pBuffer[index];
}

CFOPWideCharString CFOPWideCharString::operator + (const CFOPWideCharString& wcString)
{
	int totalLen=m_nBufferLen+wcString.m_nBufferLen;
	WCHAR* pBuf=new WCHAR[totalLen+1];

//	wcscpy(pBuf,m_pBuffer);
//	wcscat(pBuf+m_nBufferLen,wcString.m_pBuffer);

	CFOPWideCharString ret(pBuf);
	delete[] pBuf;

	return ret;
}

CFOPWideCharString& CFOPWideCharString::operator += (const CFOPWideCharString& wcString)
{
	int totalLen=m_nBufferLen+wcString.m_nBufferLen;
	WCHAR* pTempBuf=new WCHAR[totalLen+1];

//	wcscpy(pTempBuf,m_pBuffer);
//	wcscat(pTempBuf+m_nBufferLen,wcString.m_pBuffer);

	delete[] m_pBuffer;
	m_pBuffer=pTempBuf;
	m_nBufferLen=totalLen;

	return *this;
}

Status FOP_DrawString(Graphics *graphics, const TCHAR *str, const Gdiplus::Font &font, const Brush &brush, const PointF &origin)
{
	CFOPWideCharString string(str, lstrlen(str));
	return graphics->DrawString(string, wcslen(string), &font, origin, &brush);
}
Status FOP_DrawString(Graphics *graphics, const TCHAR *str, const Gdiplus::Font &font, const Brush &brush, const PointF &origin, StringFormat &fmt)
{
	CFOPWideCharString string(str, lstrlen(str));
	return graphics->DrawString(string, wcslen(string), &font, origin, &fmt, &brush);
}
Status FOP_DrawString(Graphics *graphics, const TCHAR *str, const Gdiplus::Font &font, const Brush &brush, const Point &origin)
{
	CFOPWideCharString string(str, lstrlen(str));
	return graphics->DrawString(string, wcslen(string), &font, PointF(origin.X,origin.Y), &brush);
}
Status FOP_DrawString(Graphics *graphics, const TCHAR *str, const Gdiplus::Font &font, const Brush &brush, const RectF &rect, StringFormat &fmt)
{
	CFOPWideCharString string(str, lstrlen(str));
	return graphics->DrawString(string, wcslen(string), &font, rect, &fmt, &brush);
}
Status FOP_DrawString(Graphics *graphics, const TCHAR *str, const Gdiplus::Font &font, const Brush &brush, const Rect &rect, StringFormat &fmt)
{
	CFOPWideCharString string(str, lstrlen(str));
	RectF r(rect.X, rect.Y, rect.Width, rect.Height);
	return graphics->DrawString(string, wcslen(string), &font, r, &fmt, &brush);
}

double XxnewNormAngle360(double a)
{
	while(a < 0)
	{
		a += 3600;
	}
	
	while(a >= 3600)
	{
		a -= 3600;
	}
	return a;
}

void CFOPGDIPlusProxy::DoDrawBitmap(CDC *pDC, CFOBitmap *pBitmap, CRect rcPos, 
									const int &nImageX, const int &nImageY, const int &nImageWidth, const int &nImageHeight,
									const BOOL &bImageTransparent, const COLORREF &crImageTransparent, int nRotateAngle1)
{
	double nRotateAngle = nRotateAngle1;
	nRotateAngle = XxnewNormAngle360(nRotateAngle);
	nRotateAngle = nRotateAngle /10;
	COLORREF m_crImageTransparent = crImageTransparent;
	BOOL m_bImageTransparent = bImageTransparent;
// 	FOPRect rcPos;
// 	rcPos.CreateFromCenter(rcPos1.CenterPoint(), CSize(nImageWidth, nImageHeight));
// 	if(!m_bWithGDIPlus || !gfxData.m_bDefWithGDIPlus)
// 	{
// 		CFOPVisualProxy::DoDrawBitmap(pDC, pBitmap, rcPos, nImageX, nImageY, nImageWidth, nImageHeight, m_bImageTransparent, m_crImageTransparent);
// 		return;
// 	}

	if(pBitmap->m_bAlpha && (nRotateAngle == 0))
	{
		m_crImageTransparent = pBitmap->GetPixelColor(0,0);
		m_bImageTransparent = TRUE;
		
		CBitmap *pBmp = pBitmap->m_curImageList;
		if(pBmp != NULL)
		{
			CBitmap* hbmOldGlyphs = (CBitmap *)pDCGlyphs->SelectObject (pBmp);
			
			FOPRect rcDec = rcPos;
			FOPRect rcSrc = FOPRect(CPoint(nImageX,nImageY), CSize(nImageWidth, nImageHeight));
			DrawAlpha (pDC, rcDec, 
				pDCGlyphs, rcSrc, 255);
			
			pDCGlyphs->SelectObject (hbmOldGlyphs);
			return;
		}
	}
	else if(pBitmap->m_bAlpha)
	{
		m_crImageTransparent = pBitmap->GetPixelColor(0,0);
		m_bImageTransparent = TRUE;
		
	}

	if(nRotateAngle == 0)
	{

		CFOPVisualProxy::DoDrawBitmap(pDC, pBitmap, rcPos, nImageX, nImageY, nImageWidth, nImageHeight, m_bImageTransparent, m_crImageTransparent);
		return;
	}

	Bitmap *pBitmapPlus = new Bitmap(pBitmap->m_lpBMI, pBitmap->m_lpBuf);
	
	if (NULL == pBitmapPlus)
		return;
	
	int m_nColorOffset = 1;
	ImageAttributes m_imAtt;
	if (TRUE == m_bImageTransparent)
	{
		m_imAtt.Reset();
		m_imAtt.ClearColorKey();
		
		int nR = GetRValue(m_crImageTransparent)-m_nColorOffset<0? 0:GetRValue(m_crImageTransparent)-m_nColorOffset;
		int nG = GetGValue(m_crImageTransparent)-m_nColorOffset<0? 0:GetGValue(m_crImageTransparent)-m_nColorOffset;
		int nB = GetBValue(m_crImageTransparent)-m_nColorOffset<0? 0:GetBValue(m_crImageTransparent)-m_nColorOffset;
		Color crLow((BYTE)nR, (BYTE)nG, (BYTE)nB);
		
		nR = GetRValue(m_crImageTransparent)+m_nColorOffset>255? 255:GetRValue(m_crImageTransparent)+m_nColorOffset;
		nG = GetGValue(m_crImageTransparent)+m_nColorOffset>255? 255:GetGValue(m_crImageTransparent)+m_nColorOffset;
		nB = GetBValue(m_crImageTransparent)+m_nColorOffset>255? 255:GetBValue(m_crImageTransparent)+m_nColorOffset;
		Color crHigh((BYTE)nR, (BYTE)nG, (BYTE)nB);
		
		m_imAtt.SetColorKey(crLow, crHigh);
	}
	
	ColorMatrix cm = {0,};
	cm.m[0][0] = cm.m[1][1] = cm.m[2][2] = cm.m[4][4] = 1;
	//	cm.m[3][3] = 0.5;
	cm.m[3][3] = float(m_nTransparent)/255.0f;
	m_imAtt.SetColorMatrix(&cm);
	
	RectF rcDest;
	rcDest.X = (float)rcPos.left;
	rcDest.Y = (float)rcPos.top;
	rcDest.Width = (float)rcPos.Width();
	rcDest.Height = (float)rcPos.Height();
	
	Graphics graphics(pDC->m_hDC);
	PrepareGraphics(&graphics);
	
	graphics.SetCompositingMode( CompositingModeSourceOver );  // 'Over for tranparency
	graphics.SetCompositingQuality( CompositingQualityHighQuality );
	graphics.SetPixelOffsetMode( PixelOffsetModeHighSpeed );
	graphics.SetSmoothingMode( SmoothingModeHighSpeed );              
	graphics.SetInterpolationMode( InterpolationModeHighQuality );   
// 
// 	ColorMatrix HotMat = {	1.0f, 0.00f, 0.00f, 0.00f, 0.00f,
// 									0.00f, 1.0f, 0.00f, 0.00f, 0.00f,
// 									0.00f, 0.00f, 1.0f, 0.00f, 0.00f,
// 									0.00f, 0.00f, 0.00f, float(m_nTransparent)/255.0f, 0.00f,
// 									0.0f, 0.0f, 0.0f, 0.00f, 1.00f	};
// 
// 			ImageAttributes ia;
// 			ia.SetColorMatrix(&HotMat);
	FOPPoint centerPos(rcPos.CenterPoint().x, rcPos.CenterPoint().y);
	
	if(nRotateAngle != 0)
	{
		Matrix mtr;
		mtr.Translate(centerPos.x, centerPos.y);
		mtr.Rotate(nRotateAngle);
		mtr.Translate(-centerPos.x, -centerPos.y);
		graphics.SetTransform(&mtr);

		FOPRect rcNew = rcPos;
		rcNew.CreateFromCenter(rcPos.CenterPoint(), CSize(nImageWidth, nImageHeight));

		rcDest.X = (float)rcNew.left;
		rcDest.Y = (float)rcNew.top;
		rcDest.Width = (float)rcNew.Width();
		rcDest.Height = (float)rcNew.Height();
	}
	
	graphics.DrawImage(pBitmapPlus, rcDest, (float)nImageX, (float)nImageY, (float)nImageWidth,
		(float)nImageHeight, UnitPixel, &m_imAtt);
	delete pBitmapPlus;
 
	//m_nTransparent = 255;
}
